-- Filename:     ad7813_sim.do
-- Filetype:     VHDL Source Code
-- Date:         32 sep 2012
-- Update:       -
-- Description:  VHDL Structural for encapsulation
-- Author:       J. op den Brouw
-- State:        Demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2012, De Haagse Hogeschool

-- This file contains a VHDL description of an AD7813 ADC converter, see
-- ( http://www.analog.com/en/analog-to-digital-converters/ad-converters/ad7813/products/product.html )
-- for more information.
-- The VHDL description can both be simulated and synthesized.
-- Note that in this description the CS input is tied to GND, and the signal
-- is removed from the port map.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity ad7813_sim is
	port (CLOCK_50 : in std_logic;
			BUTTON   : in std_logic_vector(2 downto 0);
			SW       : in std_logic_vector(9 downto 0);
			LEDG     : out std_logic_vector(9 downto 0);
			HEX3_D   : out std_logic_vector(6 downto 0);
			HEX2_D   : out std_logic_vector(6 downto 0);
			HEX1_D   : out std_logic_vector(6 downto 0);
			HEX0_D   : out std_logic_vector(6 downto 0);
			HEX0_DP  : out std_logic;
			HEX1_DP  : out std_logic;
			HEX2_DP  : out std_logic;
			HEX3_DP  : out std_logic;
			-- AD7813 IO, mapped to extension header
			CONVST   : in std_logic;
			-- CS is tied to GND
			--CS       : in std_logic;
			RD       : in std_logic;
			BUSY     : out std_logic;
			DB       : out std_logic_vector(7 downto 0)
		);
end entity ad7813_sim;

architecture structural of ad7813_sim is
component ad7813_module_sim is
	port (clk      : in std_logic;
			areset   : in std_logic;
			analog   : in std_logic_vector(9 downto 0);
			CONVST   : in std_logic;
			CS       : in std_logic;
			RD       : in std_logic;
			BUSY     : out std_logic;
			DB       : out std_logic_vector(7 downto 0)
		);
end component ad7813_module_sim;
constant CS : std_logic := '0';
begin

	ad7813 : ad7813_module_sim
	port map (clk => CLOCK_50,	areset => not BUTTON(0),	analog => SW,
				 CONVST => CONVST, CS => CS, RD => RD, BUSY => BUSY,
				 DB => DB);
	
	-- The unused outputs...
	HEX3_D <= (others => '1');
	HEX2_D <= (others => '1');
	HEX1_D <= (others => '1');
	HEX0_D <= (others => '1');
	
	HEX3_DP <= '1';
	HEX2_DP <= '1';
	HEX1_DP <= '1';
	HEX0_DP <= '1';
	
	LEDG<= (others => '0');
	
end architecture structural;
