-- Filename:     tb_ad7813_sim.do
-- Filetype:     VHDL Testbench
-- Date:         32 sep 2012
-- Update:       -
-- Description:  VHDL Testbench for simulation
-- Author:       J. op den Brouw
-- State:        Demo
-- Error:        -
-- Version:      1.0
-- Copyright:    (c)2012, De Haagse Hogeschool

-- This file contains a VHDL testbench for an AD7813 ADC converter, see
-- ( http://www.analog.com/en/analog-to-digital-converters/ad-converters/ad7813/products/product.html )
-- for more information.
-- The first series of statements run a Mode 1 conversion, the second
-- series run a Mode 2 conversion.

library ieee;
use ieee.std_logic_1164.all;
use ieee.numeric_std.all;

entity tb_ad7813_sim is
end entity;

architecture sim of tb_ad7813_sim is
constant small_delay : time := 1 ns;
-- The module
component ad7813_sim is
	port (CLOCK_50 : in std_logic;
			BUTTON   : in std_logic_vector(2 downto 0);
			SW       : in std_logic_vector(9 downto 0);
			LEDG     : out std_logic_vector(9 downto 0);
			HEX3_D   : out std_logic_vector(6 downto 0);
			HEX2_D   : out std_logic_vector(6 downto 0);
			HEX1_D   : out std_logic_vector(6 downto 0);
			HEX0_D   : out std_logic_vector(6 downto 0);
			HEX0_DP  : out std_logic;
			HEX1_DP  : out std_logic;
			HEX2_DP  : out std_logic;
			HEX3_DP  : out std_logic;
			-- AD7813 IO, mapped to extension header
			CONVST   : in std_logic;
			--CS       : in std_logic;
			RD       : in std_logic;
			BUSY     : out std_logic;
			DB       : out std_logic_vector(7 downto 0)
		);
end component ad7813_sim;

-- Signals to be traced, from the module
signal CLOCK_50 : std_logic;
signal BUTTON   : std_logic_vector(2 downto 0);
signal SW       : std_logic_vector(9 downto 0);
signal LEDG     : std_logic_vector(9 downto 0);
signal HEX3_D   : std_logic_vector(6 downto 0);
signal HEX2_D   : std_logic_vector(6 downto 0);
signal HEX1_D   : std_logic_vector(6 downto 0);
signal HEX0_D   : std_logic_vector(6 downto 0);
signal HEX0_DP  : std_logic;
signal HEX1_DP  : std_logic;
signal HEX2_DP  : std_logic;
signal HEX3_DP  : std_logic;
signal CONVST   : std_logic;
signal CS       : std_logic;
signal RD       : std_logic;
signal BUSY     : std_logic;
signal DB       : std_logic_vector(7 downto 0);

begin

	dut: ad7813_sim
	port map (CLOCK_50 => CLOCK_50, BUTTON => BUTTON, SW => SW, LEDG => LEDG,
				 HEX3_D => HEX3_D, HEX2_D => HEX2_D, HEX1_D => HEX1_D, HEX0_D => HEX0_D,
				 HEX0_DP => HEX0_DP, HEX1_DP => HEX1_DP, HEX2_DP => HEX2_DP, HEX3_DP => HEX3_DP,
				 CONVST => CONVST, --CS => CS, 
				 RD => RD, BUSY => BUSY, DB => DB);

	clockgen: process is
	begin
		-- give time for reset
		CLOCK_50 <= '0';
		BUTTON(0) <= '0';
		wait for 15 ns;
		BUTTON(0) <= '1';
		wait for 5 ns;
		-- forever: generate clock cycle for 20 ns and 50% d.c.
		loop
			CLOCK_50 <= '1';
			wait for 10 ns;
			CLOCK_50 <= '0';
			wait for 10 ns;
		end loop;

	end process;
	
	datagen: process is
	begin
		BUTTON(2 downto 1) <= (others => '0');
		SW <= "1111000011";
		-- CS tied to GND in schematic
		CS <= '0';
		RD <= '1';

		-- Start Mode 1 conversion
		CONVST <= '0';
		wait for 10 us;
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		CONVST <= '1';
		wait for 10 us;
		wait until CLOCK_50 = '1';
		CONVST <= '0';

		-- Wait for conversion complete
		wait until BUSY = '1';
		wait until BUSY = '0';
		wait for 1 us;
		
		-- Readout msb 8 bits
		--CS <= '0';
		RD <= '0';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 10 us;
		wait until CLOCK_50 = '1';

		-- Negate read action
		--CS <= '1';
		RD <= '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 1 us;
		wait until CLOCK_50 = '1';
		
		-- Readout lsb 2 bits
		--CS <= '0';
		RD <= '0';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 10 us;
		wait until CLOCK_50 = '1';

		-- Negate read action
		--CS <= '1';
		RD <= '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 10 us;
		wait until CLOCK_50 = '1';

		-- Start Mode 2 conversion
		CONVST <= '0';
		wait for 10 us;
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		CONVST <= '1';
		wait for 500 ns;
		wait until CLOCK_50 = '1';
		CONVST <= '0';

		-- Wait for conversion complete
		wait until BUSY = '1';
		wait until BUSY = '0';
		wait for 1 us;
		
		-- Readout msb 8 bits
		--CS <= '0';
		RD <= '0';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 10 us;
		wait until CLOCK_50 = '1';

		-- Negate read action
		--CS <= '1';
		RD <= '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 1 us;
		wait until CLOCK_50 = '1';
		
		-- Readout lsb 2 bits
		--CS <= '0';
		RD <= '0';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 10 us;
		wait until CLOCK_50 = '1';

		-- Negate read action
		--CS <= '1';
		RD <= '1';
		wait until CLOCK_50 = '1';
		wait until CLOCK_50 = '1';
		wait for 10 us;
		wait until CLOCK_50 = '1';
		
		wait;
		
	end process;
	
end architecture;